/*
 * Ext JS Library 2.0 RC 1
 * Copyright(c) 2006-2007, Ext JS, LLC.
 * licensing@extjs.com
 * 
 * http://extjs.com/license
 */

FeedViewer = {};

Ext.apply(Ext, {BLANK_IMAGE_URL: Drupal.settings.MMFV.clear});

// Work around a Firefox bug that results in only the first 4096 bytes of a
// document displaying. This is because when FF parses long XML nodes, it
// splits the data up among multiple children.
myXmlReader = Ext.extend(Ext.data.XmlReader, {
  readRecords: function(doc) {
    if(Ext.isGecko) doc.normalize();
    return myXmlReader.superclass.readRecords.call(this, doc);
  }
});

Ext.onReady(function() {
  $('#rightcolumn').hide();
  $('#tabs-container').hide();
  Ext.QuickTips.init();

  FeedViewer.SessionProvider = Ext.extend(Ext.state.Provider, {
    set: function(name, val) {
      FeedViewer.saveState(name, val);
    },
    get: function(name, val) {
      return typeof Drupal.settings.MMFV.lastState[name] == "undefined" ? val : Drupal.settings.MMFV.lastState[name];
    }
  });

  Ext.state.Manager.setProvider(new FeedViewer.SessionProvider());

  var tpl = Ext.Template.from('preview-tpl', {
    compiled:     true,
    getBody:      function(v, all) {
      return Ext.util.Format.stripScripts(v || all.description);
    },
    getEnclosure: function(v, all) {
      if( !all.enclosureUrl || !all.enclosureType ) return '';
      if( all.enclosureType == 'image/jpeg' || all.enclosureType == 'image/gif' || !Ext.IsIE && all.enclosureType == 'image/png' || Ext.IsSafari && all.enclosureType == 'image/tiff')
        return String.format('<div class="post-enclosure-img"><img src="{0}"></div>', all.enclosureUrl);
      var icon = Drupal.settings.MMFV.mediaIcons[all.enclosureType];
      icon = icon ? '<img src="' + icon + '">' : '';
      return String.format('<div class="post-enclosure-media"><a href="{0}">{1}{2}<br />({3})</a></div>', all.enclosureUrl, icon, Ext.urlDecode('zz=' + all.enclosureUrl.substr(all.enclosureUrl.lastIndexOf('/') + 1)).zz, all.enclosureType);
    },
    getAuthor:    function(v, all) {
      return all.author ? String.format('<h4 class="post-author">by {0}</h4>', all.author) : '';
    }
  });
  FeedViewer.getTemplate = function() {
    return tpl;
  };
  
  FeedViewer.saveState = function(type, val) {
    var url = Drupal.settings.MMFV.saveState + '/' + type;
    if( typeof val !== 'object' ) {
      url += '/' + val;
      val = null;
    }
    Ext.Ajax.request({
      url:     url,
      params:  val
    });
  };

  var feeds = new FeedPanel();
  var mainPanel = new MainPanel();

  feeds.on({
    feedselect: function(feed) {
      mainPanel.loadFeed(feed);
    }
  });
  
  var outerPanel = new Ext.Panel({
    id:       'panel',
    layout:   'border',
    items:    [
      feeds,
      mainPanel
    ],
    renderTo: 'feedviewer-main',
    height:   400,
    getState: function() { return { height: this.getInnerHeight() } },
    stateEvents: ['bodyresize']
  });

  var resizer = new Ext.Resizable('feedviewer-main', {
    handles:   's',
    minHeight: 400,
    pinned:    true
  });
  resizer.on('resize', function(obj, width, height, e) {
    outerPanel.setHeight(height);
    Ext.get('feedviewer-main').setWidth('auto');
  });

  $(window).resize( function() {
    outerPanel.syncSize().setWidth('auto');
  });

  for(var nid in Drupal.settings.MMFV.feeds) {
    feeds.addCategory( nid, Drupal.settings.MMFV.feeds[nid].title, Drupal.settings.MMFV.feeds[nid].expanded, Drupal.settings.MMFV.feeds[nid].readonly );
    for(var key in Drupal.settings.MMFV.feeds[nid].list) {
      var a = Ext.apply( Drupal.settings.MMFV.feeds[nid].list[key], {
        key:      key,
        readonly: Drupal.settings.MMFV.feeds[nid].readonly
      });
      feeds.addFeed(a, !a.selected, true);
    }
  }
});
