/*
 * Ext JS Library 2.0 RC 1
 * Copyright(c) 2006-2007, Ext JS, LLC.
 * licensing@extjs.com
 * 
 * http://extjs.com/license
 */

FeedWindow = function() {
  this.rssPanel = new Ext.form.FormPanel({
    labelAlign: 'top',
    title:      'RSS Feed',
    id:         'rss-panel',
    border:     false,
    autoHeight: true,
    header:     true,
    layout:     'form',
    border:     false,
    bodyStyle:  'background:transparent;',
    items:      [ this.feedUrl = new Ext.form.TextField({
      id:              'feed-url',
      fieldLabel:      'Feed URL',
      emptyText:       'http://example.com/blog/feed',
      width:           440,
      validationEvent: false,
      validateOnBlur:  false,
      msgTarget:       'under'
    }),
                  new Ext.Toolbar.TextItem('<p style="margin-bottom: 6px">Enter the URL (web address) of the page you wish to subscribe to.</p>'),
                  new Ext.Toolbar.TextItem('<p>Note: If you wish to subscribe to a password-protected Amherst page, you should use <em>Page Feed</em> instead of <em>RSS Feed.</em></p>')
    ]
  });

  this.pagePanel = new Ext.form.FormPanel({
    labelAlign: 'top',
    title:      'Page Feed',
    id:         'page-panel',
    border:     false,
    autoHeight: true,
    border:     false,
    bodyStyle:  'background:transparent;',
    items:      [ 
      this.treePanel = new Ext.tree.TreePanel({
        id:              'tree-panel',
        autoHeight:      false,
        height:          250,
        rootVisible:     false,
        containerScroll: true,
        autoScroll:      true,
        singleExpand:    true,
        loader:          new Ext.tree.TreeLoader({
          url:           Drupal.settings.MMFV.getTree,
          requestMethod: 'GET'
        }),
        root:            new Ext.tree.AsyncTreeNode({id: Drupal.settings.MMFV.treeTop})
      }),
      new Ext.Toolbar.TextItem('Find the Amherst College web page you wish to subscribe to.')
    ],
    listeners:      {
      show: function() {
        if(!this.resizer) {
          this.resizer = new Ext.Resizable(this.pagePanel.items.items[0].body, {
            handles:   's',
            wrap:      true,
            minHeight: 150,
            pinned:    true,
            listeners:      {
              resize: function() {
                this.syncShadow();
              },
              scope:     this
            }
          });
        }
      },
      scope:     this
    }
  });
  this.treePanel.getSelectionModel().on('beforeselect', function(model, node) {
    if( node.disabled ) return false;
  });

  this.tagPanel = new Ext.form.FormPanel({
    labelAlign: 'top',
    title:      'Tag Feed',
    id:         'tag-panel',
    border:     false,
    autoHeight: true,
    border:     false,
    bodyStyle:  'background:transparent;',
    hideMode:   'offsets', // Needed with "deferredRender: false" in Safari
    items:      [ this.feedTag = new Ext.form.ComboBox({
      id:              'feed-tag',
      fieldLabel:      'Tag',
      emptyText:       '(choose a tag)',
      width:           250,
      minListWidth:    250,
//      minListWidth:    70,
      validationEvent: false,
      validateOnBlur:  false,
      msgTarget:       'under',
      triggerAction:   'all',
      displayField:    'text',
      valueField:      'value',
      minChars:        1,
      forceSelection:  true,
      typeAhead:       true,
      listClass:       'feed-url',
      tpl:             new Ext.XTemplate(
          '<tpl for="."><div class="x-combo-list-item">{text}</div></tpl>'),
      store:           new Ext.data.JsonStore({
        fields: ['value', 'text'],
        url:    Drupal.settings.MMFV.tagList
      })
    }),
                  new Ext.Toolbar.TextItem('Choose a tag from the list. The list includes all tags in use on the Amherst College web site.')
    ]
  });

  this.tabPanel = new Ext.TabPanel({
    labelAlign:     'top',
    activeTab:      0,
    deferredRender: false,
    plain:          true,
    autoHeight:     true,
    bodyStyle:      'background:#DEECFD none;padding:10px;',
    items:          [ this.rssPanel, this.pagePanel, this.tagPanel ],
    listeners:      {
      tabchange: this.syncShadow,
      scope:     this
    }
  });

  this.topHelp = new Ext.FormPanel({
    labelAlign: 'top',
    items:      new Ext.Toolbar.TextItem('Choose the type of feed to add:'),
    border:     false,
    bodyStyle:  'background:transparent;padding: 5px'
  });

  this.catPanel = new Ext.FormPanel({
    labelAlign: 'top',
    border:     false,
    bodyStyle:  'background:transparent;padding:10px',
    items:      [ this.catCombo = new Ext.form.ComboBox({
      id:              'feed-cat',
      fieldLabel:      'Category',
      emptyText:       '(new category name)',
      width:           250,
      validationEvent: false,
      validateOnBlur:  false,
      msgTarget:       'under',
      triggerAction:   'all',
      displayField:    'text',
      mode:            'local',
      listClass:       'feed-url',
      tpl:             new Ext.XTemplate(
          '<tpl for="."><div class="x-combo-list-item">{text}</div></tpl>'),
      store:           this.catStore = new Ext.data.SimpleStore({
        fields: ['value', 'text']
      })
    }),
                  new Ext.Toolbar.TextItem('Create or choose a category to add the feed to (a category is simply a group of feeds). To create a new category, just type its name above.')
    ]
  });

  FeedWindow.superclass.constructor.call(this, {
    title:       'Add Feed',
    iconCls:     'feed-icon',
    id:          'add-feed-win',
    autoHeight:  true,
    width:       500,
    resizable:   false,
    plain:       true,
    modal:       true,
    autoScroll:  true,
    closeAction: 'hide',

    buttons:     [{
      text:    'Add Feed',
      handler: this.onAdd,
      scope:   this
    },{
      text:    'Cancel',
      handler: this.hide.createDelegate(this, [])
    }],

    items:       [ this.topHelp, this.tabPanel, this.catPanel ]
  });

  this.addEvents({add:true});
};

Ext.extend(FeedWindow, Ext.Window, {
  show: function(selectedCat, panel) {
    this.catCombo.setValue(selectedCat);
    var editableCats = [];
    panel.root.eachChild( function(item) {
        if(!item.attributes.readonly) {
          editableCats.push([item, item.text]);
        }
      }
    );
    this.catStore.loadData(editableCats);
    this.feedUrl.setValue('');
    this.feedTag.setValue('');
    this.panel = panel;
    FeedWindow.superclass.show.apply(this);
  },

  onAdd: function() {
    var err, params = {};
    switch(this.tabPanel.getActiveTab().id) {
      case 'rss-panel':
        params.type = 'url';
        var url = this.feedUrl.getValue();
        if( (params.data = url.trim())=='' )
            this.feedUrl.markInvalid(err='Please enter a URL.');
        break;

      case 'page-panel':
        params.type = 'cat';
        var node = this.treePanel.getSelectionModel().getSelectedNode();
        if( !node ) Ext.Msg.alert('', err='Please choose a page.');
        else params.data = node.id;
        break;

      case 'tag-panel':
        params.type = 'taxon';
        if( !(params.data = this.feedTag.getValue()) )
            this.feedTag.markInvalid(err='Please choose a tag.');
        break;
    }
    var catName = this.catCombo.getValue();
    if( (params.cat = catName.trim())=='' )
        this.catCombo.markInvalid(err='Please choose an existing category, or enter the name of a new category to create.');

    if(!err) {
      var elem;
      if(elem = this.panel.testExistingFeed.call(this.panel, params.data, params.type, params.cat)) {
        this.hide();
        Ext.Msg.alert('', 'This feed is already present in the category you chose.', function() {
            elem.parentNode.expand();
            elem.select();
            Ext.fly(elem.ui.elNode).frame();
          }
        );
      }
      else {
        this.el.mask('Adding Feed...', 'x-mask-loading');
        Ext.Ajax.request({
          url:     Drupal.settings.MMFV.addFeed,
          params:  params,
          success: this.addSuccess,
          failure: this.generalErr,
          scope:   this
        });
      }
    }
  },

  generalErr: function() {
    Ext.Msg.alert('Error', 'An error occurred when talking to the Web server.');
    this.el.unmask();
  },

  addSuccess: function(response, options) {
    var dq = Ext.DomQuery;

    var xml = response.responseXML;
    var result = xml.getElementsByTagName('result')[0];
    if(result) {
      var err = dq.selectValue('error', result);
      if(err) {
        Ext.Msg.alert('Error', err);
        return this.el.unmask();
      }
      else {
        this.el.unmask();
        this.hide();

        return this.fireEvent('validfeed',
          dq.selectValue('catKey', result), options.params.cat, {
            key:     dq.selectValue('feedKey', result),
            text:    dq.selectValue('feedName', result),
            data:    dq.selectValue('data', result),
            type:    options.params.type
          }
        );
      }
    }
    this.generalErr();
  }
});
